/// <reference lib="dom" />
import ReactDOM from 'react-dom/client';
import { getKsRendererClientUtils } from '@knapsack/renderer-webpack-base/client';
import { ReactRendererClientApp } from '@knapsack/renderer-react-components/react-renderer-client-app';
import { rendererMetaScriptTagId, } from '../types.js';
async function init() {
    try {
        const { getNeededImports, getExtraImports, rendererMeta: { disableReactStrictMode, demoWrapperProps }, rendererMethods: { getDemoApp }, } = getKsRendererClientUtils({
            rendererMetaScriptTagId,
        });
        const [neededImports, { DemoWrapper }] = await Promise.all([
            getNeededImports(),
            getExtraImports(),
        ]);
        const root = document.getElementById('render-root');
        if (!root) {
            throw new Error('No root element found');
        }
        const demoComponents = neededImports;
        const DemoApp = getDemoApp(demoComponents);
        const app = ReactRendererClientApp({
            DemoWrapper,
            DemoApp,
            disableReactStrictMode,
            demoWrapperProps,
        });
        ReactDOM.createRoot(root).render(app);
    }
    catch (e) {
        console.error(e);
        // create html element with error message
        const errorElement = document.createElement('div');
        errorElement.innerHTML = `
    <h1>Error</h1>
    <p>${e.message}</p>
  `;
        document.body.appendChild(errorElement);
    }
}
init();
//# sourceMappingURL=init.mjs.map