"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createReactPattern = void 0;
const file_utils_1 = require("@knapsack/file-utils");
const creator_utils_1 = require("@knapsack/creator-utils");
const ks_file_utils_1 = require("@knapsack/ks-file-utils");
const utils_1 = require("@knapsack/utils");
function isCapitalLetter(char) {
    return char.toUpperCase() === char;
}
function startsWithCapitalLetter(str) {
    return isCapitalLetter(str[0]);
}
function createPatternData({ importName, id, title = importName, pkgPath, initialDemoId, }) {
    return {
        id,
        title,
        description: '',
        statuses: {
            main: 'ready',
        },
        templates: [
            {
                id: 'react',
                title: 'react',
                path: pkgPath,
                alias: importName,
                templateLanguageId: 'react',
                spec: {
                    isInferred: true,
                },
                demoIds: [initialDemoId],
                blockIds: [],
            },
        ],
        tabs: [
            {
                type: 'template',
                id: 'react',
            },
        ],
        subPages: [],
    };
}
exports.createReactPattern = (0, creator_utils_1.createCreator)({
    id: 'react-patterns',
    title: 'React Ks Patterns',
    description: 'Adds React templates as Knapsack Patterns',
    getQuestions: async () => ({
        pkgPath: {
            type: 'text',
            title: 'Package path',
        },
        importPrefix: {
            type: 'text',
            title: 'Import Prefix to Remove',
        },
    }),
    getTasks: async ({ answers: { pkgPath, importPrefix = '' }, config }) => {
        const dataDir = config.dest;
        const currentPatterns = await (0, ks_file_utils_1.readKsPatternConfigs)({
            dataDir,
        });
        const currentReactPatternsImportNames = currentPatterns.reduce((cur, pattern) => {
            var _a, _b;
            const reactTemplates = (_b = (_a = pattern.templates) === null || _a === void 0 ? void 0 : _a.filter((t) => t.templateLanguageId === 'react')) !== null && _b !== void 0 ? _b : [];
            cur.push(...reactTemplates.map((t) => t.alias));
            return cur;
        }, []);
        const { exports: allImports } = await (0, file_utils_1.getJsExportNames)({
            path: pkgPath,
        });
        return [
            {
                title: 'Pick Imports to add',
                task: (_, task) => creator_utils_1.tasks.runSubCreator({
                    task,
                    config,
                    creator: (0, creator_utils_1.createCreator)({
                        id: 'react-pattern-import-names',
                        getQuestions: async () => ({
                            importNames: {
                                type: 'choices',
                                choices: allImports
                                    .filter((importName) => startsWithCapitalLetter(importName) &&
                                    !currentReactPatternsImportNames.includes(importName))
                                    .map((importName) => ({
                                    value: importName,
                                })),
                            },
                        }),
                        getTasks: async ({ answers }) => {
                            // not sure why TS failed to infer this. happened during turning on strictNullChecks
                            const { importNames } = answers;
                            const patterns = importNames.map((importName) => ({
                                importName,
                                patternId: importName.startsWith(importPrefix)
                                    ? importName.slice(importPrefix.length).toLowerCase()
                                    : importName.toLowerCase(),
                            }));
                            return [
                                ...patterns.map(({ importName, patternId }) => ({
                                    title: `Add ${importName} React Template`,
                                    task: async (__, subTask) => {
                                        const initialDemo = {
                                            type: 'data',
                                            id: (0, utils_1.makeShortId)(),
                                            title: 'Main',
                                            patternId,
                                            templateId: 'react',
                                            data: {
                                                props: {},
                                                slots: {},
                                            },
                                            meta: {
                                                createdBy: 'Knapsack',
                                                createdDate: (0, utils_1.now)(),
                                                updatedBy: 'Knapsack',
                                                updatedDate: (0, utils_1.now)(),
                                            },
                                        };
                                        const pattern = createPatternData({
                                            id: patternId,
                                            importName,
                                            pkgPath,
                                            initialDemoId: initialDemo.id,
                                        });
                                        await Promise.all([
                                            (0, ks_file_utils_1.writeDemo)({
                                                dataDir,
                                                demo: initialDemo,
                                            }),
                                            (0, ks_file_utils_1.writeKsPatternConfig)({
                                                dataDir,
                                                patternId,
                                                data: pattern,
                                            }),
                                        ]);
                                    },
                                })),
                                {
                                    title: `Updating Nav`,
                                    task: async (__, subTask) => {
                                        const { byId } = await (0, ks_file_utils_1.readKsNavConfig)({
                                            dataDir,
                                        });
                                        // attempt to find a nav group to put patterns under
                                        const componentsGroup = Object.values(byId).find(({ path, name, id }) => {
                                            // we only want groups (i.e. not pages or patterns)
                                            if (path)
                                                return false;
                                            if (name.toLowerCase() === 'patterns')
                                                return true;
                                            if (name.toLowerCase() === 'components')
                                                return true;
                                            return false;
                                        });
                                        await (0, ks_file_utils_1.addKsNavItems)({
                                            dataDir,
                                            navItems: patterns.map(({ patternId }) => ({
                                                navId: patternId,
                                                navPath: `/pattern/${patternId}`,
                                                navParent: (componentsGroup === null || componentsGroup === void 0 ? void 0 : componentsGroup.id) || 'root',
                                            })),
                                        });
                                    },
                                },
                            ];
                        },
                    }),
                }),
            },
        ];
    },
});
//# sourceMappingURL=react-creators.js.map