"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _KnapsackReactRenderer_demoWrapperPath, _KnapsackReactRenderer_disableReactStrictMode;
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnapsackReactRenderer = void 0;
const renderer_webpack_base_1 = require("@knapsack/renderer-webpack-base");
const utils_1 = require("@knapsack/utils");
const app_1 = require("@knapsack/app");
const renderers_1 = require("@knapsack/app/renderers");
const types_1 = require("@knapsack/types");
const file_utils_1 = require("@knapsack/file-utils");
const path_1 = require("path");
const utils_2 = require("./utils");
const react_creators_1 = require("./react-creators");
const types_2 = require("./types");
const { pkg } = (0, file_utils_1.findUpPkgJson)(__dirname);
class KnapsackReactRenderer extends renderer_webpack_base_1.RendererWebpackBase {
    constructor({ webpackConfig, demoWrapperPath, disableReactStrictMode, codeSrcs = [], } = {}) {
        super({
            id: 'react',
            language: 'jsx',
            webpackConfig,
            codeSrcsUserConfig: codeSrcs,
            extraScripts: [
                // this is the code in `./client/init.mts`
                (0, path_1.join)(__dirname, '../dist/client/init.mjs'),
            ],
        });
        _KnapsackReactRenderer_demoWrapperPath.set(this, void 0);
        _KnapsackReactRenderer_disableReactStrictMode.set(this, void 0);
        this.setConfig = (opt) => {
            super.setConfig(opt);
            const { absolutePath, exists: demoWrapperPathExists, type, } = this.resolvePathSync({
                path: __classPrivateFieldGet(this, _KnapsackReactRenderer_demoWrapperPath, "f"),
                resolveFromDir: this.userConfigDir,
            });
            if (!demoWrapperPathExists) {
                throw new Error(`Could not find demo wrapper at: "${__classPrivateFieldGet(this, _KnapsackReactRenderer_demoWrapperPath, "f")}"\nPlease adjust setting in "knapsack.config.js" or pass a different path when creating the React Renderer.`);
            }
            this.registerExtraJsImports({
                DemoWrapper: {
                    exportName: 'DemoWrapper',
                    isDefaultExport: true,
                    path: type === 'package' ? __classPrivateFieldGet(this, _KnapsackReactRenderer_demoWrapperPath, "f") : absolutePath,
                },
            });
        };
        this.init = async (opt) => {
            await super.init(opt);
            this.assets = await (0, utils_2.copyReactAssets)(this.outputDir, this.publicPath);
        };
        this.hydrate = async (opt) => {
            await super.hydrate(opt);
            await this.init({ missingFileVerbosity: 'silent' });
        };
        this.getMeta = () => ({
            id: this.id,
            title: 'React',
            aliasUse: 'optional',
            aliasTitle: 'Named Export',
            aliasIsJsNamedExport: true,
            aliasDescription: 'If `export X` was used instead of `export default`, then provide X.',
            enableDataDemos: true,
            enableTemplateDemos: true,
            hasSlotsSupport: true,
            hasSlotOptionsSupport: true,
            version: pkg.version,
            hasInferSpecSupport: true,
            syntaxHighlightingLanguage: 'jsx',
            hasTemplateSuggestionsSupport: true,
            prototypingTemplate: {
                path: '@knapsack/renderer-react/prototype-template',
                spec: {
                    isInferred: false,
                    props: {
                        type: 'object',
                        properties: {},
                    },
                    slots: {
                        children: { title: 'Children' },
                    },
                },
            },
        });
        this.changeCase = (str) => (0, utils_1.pascalCase)(str);
        this.createWebpackConfig = () => {
            const config = super.createWebpackConfig();
            config.externals = {
                react: 'React',
                'react-dom': 'ReactDOM',
            };
            return config;
        };
        this.render = async (opt) => {
            var _a;
            if (((_a = opt.demo) === null || _a === void 0 ? void 0 : _a.type) === 'template') {
                const { absolutePath: templateDemoPath, exists } = await this.resolvePath(opt.demo.templateInfo.path);
                if (!exists) {
                    throw new Error(`Template demo file does not exist: "${opt.demo.templateInfo.path}"`);
                }
                const [templateFileContents, { usage, importMap }] = await Promise.all([
                    (0, file_utils_1.readFile)(templateDemoPath),
                    this.getUsageAndImports({
                        ...opt,
                        importMap: new Map(),
                    }),
                ]);
                const demoApp = await (0, utils_2.getDemoAppUsage)({
                    children: usage,
                });
                const results = await this.prepClientRenderResults({
                    usage: templateFileContents,
                    demoApp,
                    renderOptions: opt,
                    importMap,
                });
                return results;
            }
            if ((0, types_1.isDemoWithData)(opt.demo)) {
                const { usage, importMap } = await this.getUsageAndImports({
                    ...opt,
                    importMap: new Map(),
                });
                const importCode = renderer_webpack_base_1.RendererWebpackBase.createJsImportCodeBlock({
                    importMap,
                });
                const [demoAppUsage, demoApp] = await Promise.all([
                    (0, utils_2.getDemoAppUsage)({
                        children: usage,
                        imports: importCode,
                        format: true,
                    }),
                    (0, utils_2.getDemoAppUsage)({
                        children: usage,
                    }),
                ]);
                return this.prepClientRenderResults({
                    demoApp,
                    usage: demoAppUsage,
                    renderOptions: opt,
                    importMap,
                });
            }
            const _exhaustiveCheck = opt.demo;
            throw new Error(`Unhandled demo type at reactRenderer.render`);
        };
        this.getUsageAndImports = async ({ demo, state, importMap, }) => {
            if (!demo) {
                throw new Error(`No demo provided while rendering `);
            }
            if ((0, types_1.isDemoWithData)(demo)) {
                const { data: { props, slots, slotsOptionsComputed }, } = demo;
                const { codeSrcPath, exportName } = this.normalizeDemo({
                    demo,
                    state,
                });
                if (!codeSrcPath) {
                    throw new Error(`Could not determine code source path for demo: ${JSON.stringify(demo)}`);
                }
                const { name: templateName } = await this.addUniqueValueToImportMap({
                    importMap,
                    path: codeSrcPath,
                    alias: exportName,
                });
                const children = [];
                const extraProps = [];
                if (slots) {
                    const slotNames = Object.keys(slots);
                    const slotUsages = await Promise.all(slotNames.map(async (slotName) => {
                        const slotItems = slots[slotName];
                        const slotItemsUsages = await Promise.all(slotItems
                            // remove any incomplete item - happens when users are filling out form
                            .filter((slotItem) => {
                            if (!slotItem)
                                return false;
                            if (slotItem.type !== 'text') {
                                if (slotItem.type === 'template-demo' && !slotItem.demoId) {
                                    return false;
                                }
                            }
                            return true;
                        })
                            .map(async (slotItem) => {
                            var _a;
                            if (slotItem.type === 'text') {
                                if (slotItems.length === 1 && slotName !== 'children') {
                                    // The ` prevents multi-line strings from causing error (even though they don't show a line break)
                                    return `\`${slotItem.text}\``;
                                }
                                return slotItem.text;
                            }
                            if (slotItem.type === 'template-reference') {
                                const slottedTemplate = (_a = state.patterns[slotItem.patternId]) === null || _a === void 0 ? void 0 : _a.templates.find((t) => t.id === slotItem.templateId);
                                if (!slottedTemplate) {
                                    throw new Error(`Could not find slotted template: ${slotItem.patternId} ${slotItem.templateId}`);
                                }
                                const pathInfo = (0, file_utils_1.getPathType)(slottedTemplate.path);
                                if (pathInfo.type === 'absolute') {
                                    throw new Error(`Absolute paths are not supported for slotted templates: ${slottedTemplate.path}`);
                                }
                                const templateRefImport = await this.addUniqueValueToImportMap({
                                    importMap,
                                    path: pathInfo.path,
                                    alias: slottedTemplate.alias || 'default',
                                });
                                /**
                                 * i.e. Given a React Component, `Button`, normally this would be `<Button>` with a demo, but since there is none this will just be a reference to it: `Button`
                                 * @see {KsSlotInfo['isTemplateReference']}
                                 * @see {SlottedTemplate}
                                 */
                                return templateRefImport.name;
                            }
                            if (slotItem.type === 'template-demo') {
                                const thisDemo = slotItem.demo;
                                if (!thisDemo) {
                                    throw new Error(`Could not find slotted template demo ${JSON.stringify(slotItem)}`);
                                }
                                const { usage } = await this.getUsageAndImports({
                                    demo: thisDemo,
                                    state,
                                    importMap,
                                });
                                return usage;
                            }
                            const _exhaustiveCheck = slotItem;
                            throw new Error(`Unknown slot item: ${JSON.stringify(slotItem)}`);
                        }));
                        return {
                            slotName,
                            slotItemsUsages,
                        };
                    }));
                    slotUsages.forEach(({ slotName, slotItemsUsages }) => {
                        const slotOptionsComputed = slotsOptionsComputed === null || slotsOptionsComputed === void 0 ? void 0 : slotsOptionsComputed[slotName];
                        const { openTag, closeTag } = (0, renderers_1.createSlotOptionsHtmlTags)({
                            slotOptionsComputed,
                            classAttributeName: 'className',
                            stylesValueType: 'object',
                        });
                        if (openTag) {
                            if (slotName === 'children') {
                                children.push(openTag);
                            }
                            else {
                                extraProps.push({
                                    key: slotName,
                                    value: openTag,
                                });
                            }
                        }
                        slotItemsUsages.forEach((usage) => {
                            if (slotName === 'children') {
                                children.push(usage);
                            }
                            else {
                                extraProps.push({
                                    key: slotName,
                                    value: usage,
                                });
                            }
                        });
                        if (closeTag) {
                            if (slotName === 'children') {
                                children.push(closeTag);
                            }
                            else {
                                extraProps.push({
                                    key: slotName,
                                    value: closeTag,
                                });
                            }
                        }
                    });
                }
                const usage = await (0, utils_2.getUsage)({
                    templateName,
                    props,
                    children: children.join('\n'),
                    extraProps,
                });
                return {
                    usage,
                    importMap,
                };
            }
            if (demo.type === 'template') {
                const { templateInfo } = demo;
                const pathInfo = (0, file_utils_1.getPathType)(templateInfo.path);
                if (pathInfo.type === 'absolute') {
                    throw new Error(`Absolute paths are not supported for templates: ${templateInfo.path}`);
                }
                const { name: templateName } = await this.addUniqueValueToImportMap({
                    importMap,
                    path: pathInfo.path,
                    alias: templateInfo.alias || 'default',
                });
                const usage = await (0, utils_2.getUsage)({ templateName });
                return {
                    usage,
                    importMap,
                };
            }
            const _exhaustiveCheck = demo;
            throw new Error(`Unhandled demo type at reactRenderer.getUsageAndImports`);
        };
        this.getDiscovery = async () => {
            const codeSrcPaths = this.getCodeSrcs().map(({ path }) => path);
            const results = await Promise.all(codeSrcPaths.map(async (path) => {
                try {
                    const { exports, errorMsg } = await (0, file_utils_1.getJsExportNames)({
                        path,
                        resolveFromDir: this.dataDir,
                        pkgPathAliases: this.pkgPathAliases,
                    });
                    if (errorMsg) {
                        throw new Error(errorMsg);
                    }
                    return {
                        path,
                        exportNames: exports
                            // only allow names that start with a capital letter (i.e. React Component names)
                            // will exclude helper functions and custom hooks
                            .filter((e) => e === 'default' || (0, utils_1.isFirstLetterCapital)(e)),
                    };
                }
                catch (e) {
                    this.log.warn(`Could not get import names for "${path}": ${e.message}`, null, this.logPrefix);
                    return { path, exportNames: [] };
                }
            }));
            return {
                rendererId: this.id,
                meta: this.getMeta(),
                codeSrcsByPath: Object.fromEntries(this.codeSrcs.entries()),
                templates: results.flatMap(({ path, exportNames }) => exportNames.map((exportName) => (0, types_1.addIdToTemplateInfo)({
                    rendererId: this.id,
                    codeSrcPath: path,
                    exportName,
                }))),
            };
        };
        this.inspect = async (templateInfo) => {
            const { codeSrcPath, exportName } = templateInfo;
            const { inferSpec } = await import('./analyze/index.mjs');
            const { ksTemplateSpec: spec } = await inferSpec({
                path: codeSrcPath,
                namedExport: exportName,
                resolveFrom: this.dataDir,
                pkgPathAliases: this.pkgPathAliases,
            });
            return {
                type: 'success',
                spec,
            };
        };
        this.watch = async () => {
            super.watch();
            app_1.knapsackEvents.onPatternTemplateChanged(() => {
                (0, utils_2.clearInferSpecCache)();
            });
        };
        this.getTemplateMeta = async ({ pattern, template, }) => {
            const files = [];
            if ((0, types_1.isSpecNotInferred)(template.spec)) {
                const schema = JSON.parse(JSON.stringify(template.spec.props || {}));
                if (template.spec.slots) {
                    Object.entries(template.spec.slots).forEach(([id, info]) => {
                        var _a;
                        schema.properties[id] = {
                            typeof: 'function',
                            tsType: 'React.ReactNode',
                            description: info.allowedPatternIds
                                ? `${info.description}. Only use: ${info.allowedPatternIds.join(', ')}`
                                : info.description,
                        };
                        schema.required = (_a = schema.required) !== null && _a !== void 0 ? _a : [];
                        if (info.isRequired)
                            schema.required.push(id);
                    });
                }
                const typeDefs = await KnapsackReactRenderer.convertSchemaToTypeScriptDefs({
                    schema,
                    title: `${this.changeCase(pattern.id)}Props`,
                    // @todo pull in base url
                    description: `[Knapsack Docs](http://localhost:3999/pattern/${pattern.id}/${template.id})`,
                    patternId: pattern.id,
                    templateId: template.id,
                    postBanner: `import * as React from 'react';`,
                });
                files.push({
                    contents: typeDefs,
                    encoding: 'utf8',
                    path: `${pattern.id}.${template.id}.spec.d.ts`,
                });
                files.push({
                    contents: JSON.stringify(schema, null, '  '),
                    encoding: 'utf8',
                    path: `${pattern.id}.${template.id}.spec.json`,
                });
            }
            return files;
        };
        this.alterTemplateMetaFiles = async ({ files, metaDir, }) => {
            const imports = [];
            const ext = '.spec.d.ts';
            files.forEach((file) => {
                if (file.path.endsWith(ext)) {
                    const { base } = (0, path_1.parse)(file.path);
                    const [patternId, templateId] = base.split('.');
                    const isFirst = templateId === this.id;
                    const exportName = this.changeCase(`${patternId}Props`);
                    const exportNameWithTemplateId = this.changeCase(`${patternId}-${templateId}Props`);
                    imports.push(`export { ${isFirst
                        ? exportName
                        : `${exportName} as ${exportNameWithTemplateId}`} } from './${(0, path_1.relative)(metaDir, file.path).replace('.d.ts', '')}';`);
                }
            });
            // extra line at end of file
            imports.push('');
            return [
                ...files,
                {
                    contents: imports.join('\n'),
                    encoding: 'utf8',
                    path: (0, path_1.join)(metaDir, 'react.d.ts'),
                },
            ];
        };
        this.getTemplateSuggestions = async ({ newPath, state, }) => {
            // Deduplicate suggestions using Map with stringified path/alias as keys.
            // Map values are suggestion objects, Array.from(map.values()) returns deduplicated array.
            const usedSuggestions = Array.from(new Map(Object.values(state.patterns).reduce((acc, { templateDemos, templates }) => {
                templates.forEach(({ path, alias, templateLanguageId }) => {
                    if (templateLanguageId !== this.id)
                        return;
                    acc.push([JSON.stringify({ path, alias }), { path, alias }]);
                });
                templateDemos.forEach(({ templateInfo: { path, alias }, templateLanguageId }) => {
                    if (templateLanguageId !== this.id)
                        return;
                    acc.push([JSON.stringify({ path, alias }), { path, alias }]);
                });
                return acc;
            }, [])).values());
            const codeSrcs = new Set(this.getCodeSrcs().map(({ path }) => path));
            const allPaths = [
                ...new Set([
                    newPath,
                    ...Object.keys(this.pkgPathAliases || {}),
                    ...codeSrcs,
                ]),
            ];
            // @todo use getDiscovery
            const allSuggestions = await Promise.all(allPaths.map(async (path) => {
                if (!path)
                    return [];
                try {
                    const { exports, errorMsg } = await (0, file_utils_1.getJsExportNames)({
                        path,
                        resolveFromDir: this.dataDir,
                        pkgPathAliases: this.pkgPathAliases,
                    });
                    if (errorMsg) {
                        throw new Error(errorMsg);
                    }
                    return (exports
                        // only allow names that start with a capital letter (i.e. React Component names)
                        // will exclude helper functions and custom hooks
                        .filter((e) => e === 'default' || (0, utils_1.isFirstLetterCapital)(e))
                        .map((name) => ({
                        alias: name,
                        path,
                    })));
                }
                catch (e) {
                    // only want this showing with verbose since template suggestions are not critical
                    this.log.verbose(`Error getting import names for ${path}: ${e.message}`, null, this.logPrefix);
                    return [];
                }
            })).then((results) => {
                // sadly `Promise.all` & `.flatMap` just don't work, so this:
                return results.flat();
            });
            const suggestions = allSuggestions.filter((s) => {
                return !usedSuggestions.find((us) => us.alias === s.alias && us.path === s.path);
            });
            const connectedSuggestions = usedSuggestions.map((s) => ({
                ...s,
                connected: true,
            }));
            return {
                suggestions: [...suggestions, ...connectedSuggestions],
            };
        };
        this.language = 'jsx';
        this.assets = [];
        __classPrivateFieldSet(this, _KnapsackReactRenderer_disableReactStrictMode, disableReactStrictMode, "f");
        this.creators = [react_creators_1.createReactPattern];
        if (demoWrapperPath) {
            __classPrivateFieldSet(this, _KnapsackReactRenderer_demoWrapperPath, demoWrapperPath, "f");
        }
        else {
            const defaultDemoWrapperPath = '@knapsack/renderer-react-components/demo-wrapper';
            const { absolutePath, exists } = this.resolvePathSync({
                path: defaultDemoWrapperPath,
                // we need to resolve this from this file since the demo wrapper's package is a dependency of this package
                // if we resolved it from `this.userConfigDir` then the path would be incorrect for NPM installs that do not hoist sub-dependencies to the top-level
                resolveFromDir: __dirname,
            });
            if (!exists) {
                throw new Error(`Could not locate the default Demo Wrapper component at "${defaultDemoWrapperPath}". This is an internal error and may be caused by a NPM install issue.`);
            }
            __classPrivateFieldSet(this, _KnapsackReactRenderer_demoWrapperPath, absolutePath, "f");
        }
    }
    async prepClientRenderResults({ usage, demoApp, importMap, renderOptions: { demo }, }) {
        const meta = {
            demo,
            disableReactStrictMode: __classPrivateFieldGet(this, _KnapsackReactRenderer_disableReactStrictMode, "f"),
            neededImportsByCodeSrcId: (0, renderer_webpack_base_1.convertImportMapToNeededImportsByCodeSrcId)(importMap),
            demoWrapperProps: {
                demo,
            },
        };
        let code = `
    window.knapsack = window.knapsack || {};
    window.knapsack.getDemoApp = ({ ${[...importMap.keys()].join(', ')} }) => {
      ${demoApp}
      return ${utils_2.demoAppName}
    }
    `;
        let errorHtmlMsg = '';
        try {
            code = await (0, renderers_1.babelCodeForBrowser)({ code });
        }
        catch (e) {
            console.log(code);
            console.log('---original code before babel error ^---');
            console.trace(e.message);
            code = `console.error(${JSON.stringify(e.message)});`;
            errorHtmlMsg = `<pre><code>${e.message}</code></pre>`;
        }
        const html = `
    <script type="application/json" id="${types_2.rendererMetaScriptTagId}">${JSON.stringify(meta)}</script>
    <script type="application/javascript">${code}</script>
    <div id="render-root" class="knapsack-pattern-direct-parent" data-dev-note="Knapsack React Template Wrapper"></div>
    ${this.assets.map((asset) => `<script src="${asset}"></script>`).join('\n')}
    ${renderer_webpack_base_1.RendererWebpackBase.createHtmlTagsForAssetPaths({
            assets: this.getWebpackAssetPaths(),
            // we need the scripts to finish adding methods to the global knapsack object synchronously before the client-side code runs that is in the <script> tag below
            scriptTagsAreAsync: false,
        })}
    ${errorHtmlMsg}
    `;
        return {
            ok: !errorHtmlMsg,
            html: await (0, file_utils_1.formatCode)({
                contents: html,
                path: 'x.html', // doing this to set format language
            }),
            usage,
            templateLanguage: this.language,
        };
    }
}
exports.KnapsackReactRenderer = KnapsackReactRenderer;
_KnapsackReactRenderer_demoWrapperPath = new WeakMap(), _KnapsackReactRenderer_disableReactStrictMode = new WeakMap();
//# sourceMappingURL=renderer-react.js.map